package com.huashe.park.core.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Param;

import com.huashe.common.annotation.mybatis.Tenant;
import com.huashe.park.domain.dto.cons.PileRealtimeIndex;
import com.huashe.park.domain.entity.ConsPileHoleInfo;

/**
 * 施工桩点信息Mapper接口
 *
 * @author ruoyi
 * @date 2025 -02-13
 */
public interface ConsPileHoleInfoMapper {
    /**
     * 查询施工桩点信息
     *
     * @param id 施工桩点信息主键
     * @return 施工桩点信息 cons pile hole info
     */
    public ConsPileHoleInfo selectConsPileHoleInfoById(Long id);

    public ConsPileHoleInfo selectConsPileHoleInfoByByteId(String id);

    /**
     * 查询施工桩点信息列表
     *
     * @param consPileHoleInfo 施工桩点信息
     * @return 施工桩点信息集合 list
     */
    public List<ConsPileHoleInfo> selectConsPileHoleInfoList(ConsPileHoleInfo consPileHoleInfo);

    @Tenant(tableAlias = "cphi")
    List<PileRealtimeIndex> selectConsPileHoleRealtimeIndex(ConsPileHoleInfo consPileHoleInfo);

    public List<ConsPileHoleInfo> selectNonConsPileHoleInfo(ConsPileHoleInfo consPileHoleInfo);

    /**
     * 新增施工桩点信息
     *
     * @param consPileHoleInfo 施工桩点信息
     * @return 结果 int
     */
    public int insertConsPileHoleInfo(ConsPileHoleInfo consPileHoleInfo);

    /**
     * Batch insert cons pile hole info int.
     *
     * @param consPileHoleInfos the cons pile hole infos
     * @return the int
     */
    public int batchInsertConsPileHoleInfo(@Param("list") List<ConsPileHoleInfo> consPileHoleInfos);

    /**
     * 修改施工桩点信息
     *
     * @param consPileHoleInfo 施工桩点信息
     * @return 结果 int
     */
    public int updateConsPileHoleInfo(ConsPileHoleInfo consPileHoleInfo);

    /**
     * 删除施工桩点信息
     *
     * @param id 施工桩点信息主键
     * @return 结果 int
     */
    public int deleteConsPileHoleInfoById(Long id);

    /**
     * 批量删除施工桩点信息
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果 int
     */
    public int deleteConsPileHoleInfoByIds(Long[] ids);

    /**
     * Delete cons pile hole info by cons unit id.
     *
     * @param consUnitId the cons unit id
     */
    void deleteConsPileHoleInfoByConsUnitId(Long consUnitId);
}
