/*
 * 文 件 名:  CollectConfig
 * 版    权:
 * 描    述:  <描述>
 * 修 改 人:  learshaw
 * 修改时间:  2021/9/22
 * 跟踪单号:  <跟踪单号>
 * 修改单号:  <修改单号>
 * 修改内容:  <修改内容>
 */
package com.huashe.common.data.config;

import com.huashe.common.data.service.RegionService;
import com.huashe.common.data.service.WeatherColService;
import com.huashe.common.exception.BusinessException;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 采集业务配置
 * <功能详细描述>
 *
 * @author learshaw
 * @version [版本号, 2021/9/22]
 * @see [相关类/方法]
 * @since [产品/模块版本]
 */
@Configuration
public class CollectConfig {
    @Value("${weather.service-provider}")
    private String weatherServiceProvider;

    @Value("${weather.region.conf-adcodes}")
    private String weatherAdcodes;

    @Value("${weather.region.source}")
    private String weatcherAdcodeSource;

    @Value("${holiday.url}")
    private String holidayColUrl;

    /**
     * 区域映射装配
     */
    @Resource
    private RegionService regionService;

    @Bean(value = "weatherColService")
    public WeatherColService weatherColService(
        @Qualifier(value = "weatherColAmapService") WeatherColService amapColService) {
        if (StringUtils.equals("amap", weatherServiceProvider)) {
            return amapColService;
        }
        else {
            throw new BusinessException(-1, "serviceProvider 未匹配!");
        }
    }

    /**
     * 获取天气采集目标行政区划
     * @return
     */
    public List<String> getWeatherAdcodes(){
        if (StringUtils.equals("db", weatcherAdcodeSource)) {
            return regionService.getPrefectureLevelCity();
        }
        else {
            String[] areaArray = null;

            if (StringUtils.isBlank(weatherAdcodes)) {
                areaArray = StringUtils.split(weatherAdcodes, ",");
            }

            return ArrayUtils.isNotEmpty(areaArray) ? Arrays.asList(areaArray) : new ArrayList<>();
        }
    }

    public String getHolidayColUrl() {
        return holidayColUrl;
    }
}