/*
 * 文 件 名:  HttpUtils
 * 版    权:
 * 描    述:  <描述>
 * 修 改 人:  learshaw
 * 修改时间:  2021/9/17
 * 跟踪单号:  <跟踪单号>
 * 修改单号:  <修改单号>
 * 修改内容:  <修改内容>
 */
package com.huashe.common.data.util;

import com.huashe.common.exception.Assert;
import com.huashe.common.exception.BusinessException;
import com.huashe.common.utils.CloseUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * HttpUtils
 * <功能详细描述>
 *
 * @author learshaw
 * @version [版本号, 2021/9/17]
 * @see [相关类/方法]
 * @since [产品/模块版本]
 */
public class HttpUtils {
    private static final Logger log = LoggerFactory.getLogger(HttpUtils.class);

    private static final int CONNECT_TIMEOUT = 60000;

    private static final int REQUEST_TIMEOUT = 30000;

    private static final int SOCKET_TIMEOUT = 30000;

    /**
     * 发送Get请求
     * @param url 地址
     * @return 响应
     */
    public static String doGet(String url) {
        CloseableHttpClient httpClient = null;
        CloseableHttpResponse response = null;
        String result = null;

        try {
            Assert.notEmpty(url, -1, "url 地址不合法!");
            httpClient = HttpClients.createDefault();
            HttpGet httpGet = new HttpGet(url);
            httpGet.setHeader("Content-Type", "application/json;charset=UTF-8");
            httpGet.setConfig(buildRequestConfig());
            response = httpClient.execute(httpGet);
            HttpEntity entity = response.getEntity();
            result = EntityUtils.toString(entity);
        }
        catch (BusinessException var11) {
            log.error("doGet fail! {}", var11.getMessage());
            throw var11;
        }
        catch (Exception var12) {
            log.error("doGet fail!", var12);
            throw new BusinessException(-1, var12.getMessage(), var12);
        }
        finally {
            CloseUtils.close(response);
            CloseUtils.close(httpClient);
        }

        return result;
    }

    /**
     * 发送post请求
     * @param url 地址
     * @param payload 请求参数
     * @return
     */
    public String doPost(String url, String payload) {
        CloseableHttpClient httpClient = null;
        CloseableHttpResponse httpResponse = null;
        String result = null;

        try {
            Assert.notEmpty(url, -1, "url 地址不合法!");
            httpClient = HttpClients.createDefault();
            HttpPost httpPost = new HttpPost(url);
            httpPost.setConfig(buildRequestConfig());
            httpPost.addHeader("Content-Type", "application/json;charset=UTF-8");
            httpPost.setEntity(new StringEntity(payload));

            httpResponse = httpClient.execute(httpPost);
            HttpEntity entity = httpResponse.getEntity();
            result = EntityUtils.toString(entity);
        }
        catch (BusinessException var11) {
            log.error("doPost fail! {}", var11.getMessage());
            throw var11;
        }
        catch (Exception var12) {
            log.error("doPost fail!", var12);
            throw new BusinessException(-1, var12.getMessage(), var12);
        }
        finally {
            CloseUtils.close(httpResponse);
            CloseUtils.close(httpClient);
        }

        return result;
    }

    private static RequestConfig buildRequestConfig() {
        return RequestConfig.custom().setConnectTimeout(CONNECT_TIMEOUT).setConnectionRequestTimeout(REQUEST_TIMEOUT)
            .setSocketTimeout(SOCKET_TIMEOUT).build();
    }
}