package com.ruoyi.web.controller.ems;

import com.huashe.common.domain.AjaxResult;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.ems.domain.AdmOpAlarmPolicy;
import com.ruoyi.ems.service.IAdmOpAlarmPolicyService;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 能源设施告警策略Controller
 *
 * @author ruoyi
 * @date 2024-08-26
 */
@RestController
@RequestMapping("/ems/alarm")
@Api(value = "AdmOpAlarmPolicyController", description = "能源设施告警策略")
public class AdmOpAlarmPolicyController extends BaseController
{
    @Autowired
    private IAdmOpAlarmPolicyService opAlarmPolicyService;

    /**
     * 查询能源设施告警策略列表
     */
    @PreAuthorize("@ss.hasPermi('ems:alarm-strategy:list')")
    @GetMapping("/list")
    public TableDataInfo list(AdmOpAlarmPolicy opAlarmPolicy)
    {
        startPage();
        List<AdmOpAlarmPolicy> list = opAlarmPolicyService.selectAdmOpAlarmPolicyList(opAlarmPolicy);
        return getDataTable(list);
    }

    /**
     * 导出能源设施告警策略列表
     */
    @PreAuthorize("@ss.hasPermi('ems:alarm-strategy:export')")
    @Log(title = "能源设施告警策略", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, AdmOpAlarmPolicy opAlarmPolicy)
    {
        List<AdmOpAlarmPolicy> list = opAlarmPolicyService.selectAdmOpAlarmPolicyList(opAlarmPolicy);
        ExcelUtil<AdmOpAlarmPolicy> util = new ExcelUtil<AdmOpAlarmPolicy>(AdmOpAlarmPolicy.class);
        util.exportExcel(response, list, "能源设施告警策略数据");
    }

    /**
     * 获取能源设施告警策略详细信息
     */
    @PreAuthorize("@ss.hasPermi('ems:alarm-strategy:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(opAlarmPolicyService.selectAdmOpAlarmPolicyById(id));
    }

    /**
     * 新增能源设施告警策略
     */
    @PreAuthorize("@ss.hasPermi('ems:alarm-strategy:add')")
    @Log(title = "能源设施告警策略", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody AdmOpAlarmPolicy opAlarmPolicy)
    {
        return toAjax(opAlarmPolicyService.insertAdmOpAlarmPolicy(opAlarmPolicy));
    }

    /**
     * 修改能源设施告警策略
     */
    @PreAuthorize("@ss.hasPermi('ems:alarm-strategy:edit')")
    @Log(title = "能源设施告警策略", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody AdmOpAlarmPolicy opAlarmPolicy)
    {
        return toAjax(opAlarmPolicyService.updateAdmOpAlarmPolicy(opAlarmPolicy));
    }

    /**
     * 删除能源设施告警策略
     */
    @PreAuthorize("@ss.hasPermi('ems:alarm-strategy:remove')")
    @Log(title = "能源设施告警策略", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(opAlarmPolicyService.deleteAdmOpAlarmPolicyByIds(ids));
    }
}
