package com.ruoyi.web.controller.ems;

import com.huashe.common.domain.AjaxResult;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.ems.domain.EmsFacs;
import com.ruoyi.ems.service.IEmsFacsService;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 能源设施/系统Controller
 *
 * @author ruoyi
 * @date 2024-07-10
 */
@RestController
@RequestMapping("/ems/basecfg/emsfacs")
@Api(value = "EmsFacsController", description = "能源设施管理")
public class EmsFacsController extends BaseController {
    @Autowired
    private IEmsFacsService emsFacsService;

    /**
     * 查询能源设施/系统列表
     */
    @PreAuthorize("@ss.hasPermi('basecfg:emsfacs:list')")
    @GetMapping("/list")
    public TableDataInfo list(EmsFacs emsFacs) {
        startPage();
        List<EmsFacs> list = emsFacsService.selectEmsFacsList(emsFacs);
        return getDataTable(list);
    }

    /**
     * 导出能源设施/系统列表
     */
    @PreAuthorize("@ss.hasPermi('basecfg:emsfacs:export')")
    @Log(title = "能源设施/系统", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EmsFacs emsFacs) {
        List<EmsFacs> list = emsFacsService.selectEmsFacsList(emsFacs);
        ExcelUtil<EmsFacs> util = new ExcelUtil<EmsFacs>(EmsFacs.class);
        util.exportExcel(response, list, "能源设施/系统数据");
    }

    /**
     * 获取能源设施/系统详细信息
     */
    @PreAuthorize("@ss.hasPermi('basecfg:emsfacs:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(emsFacsService.selectEmsFacsById(id));
    }

    /**
     * 新增能源设施/系统
     */
    @PreAuthorize("@ss.hasPermi('basecfg:emsfacs:add')")
    @Log(title = "能源设施/系统", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EmsFacs emsFacs) {
        return toAjax(emsFacsService.insertEmsFacs(emsFacs));
    }

    /**
     * 修改能源设施/系统
     */
    @PreAuthorize("@ss.hasPermi('basecfg:emsfacs:edit')")
    @Log(title = "能源设施/系统", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EmsFacs emsFacs) {
        return toAjax(emsFacsService.updateEmsFacs(emsFacs));
    }

    /**
     * 删除能源设施/系统
     */
    @PreAuthorize("@ss.hasPermi('basecfg:emsfacs:remove')")
    @Log(title = "能源设施/系统", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(emsFacsService.deleteEmsFacsByIds(ids));
    }

    /**
     * 获取所有能源设施/系统列表
     *
     * @return 能源设施/系统列表
     */
    @GetMapping("/listall")
    public AjaxResult listAll(@RequestParam(name = "refArea", required = false) String refArea,
        @RequestParam(name = "facsCategory", required = false) String facsCategory,
        @RequestParam(name = "subCategory", required = false) String subCategory) {
        EmsFacs emsFacs = new EmsFacs();
        emsFacs.setRefArea(refArea);
        emsFacs.setFacsCategory(facsCategory);
        emsFacs.setFacsSubCategory(subCategory);

        return success(emsFacsService.selectEmsFacsList(emsFacs));
    }

    /**
     * 网-服务区日功率指标
     *
     * @param areaCode
     * @return
     */
    @GetMapping("/area/day/power/index")
    public AjaxResult qryAreaDayPowerIndex(@RequestParam("areaCode") String areaCode) {
        return success(emsFacsService.qryAreaDayPowerIndex(areaCode));
    }
}
