package com.ruoyi.ems.controller;

import com.ruoyi.common.core.utils.poi.ExcelUtil;
import com.ruoyi.common.core.web.controller.BaseController;
import com.huashe.common.domain.AjaxResult;
import com.ruoyi.common.core.web.page.TableDataInfo;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import com.ruoyi.ems.domain.AdmEmsElecPgIndex;
import com.ruoyi.ems.service.IAdmEmsElecPgIndexService;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 电网设施指标Controller
 * 
 * @author ruoyi
 * @date 2024-08-30
 */
@RestController
@RequestMapping("/elecPgIndex")
@Api(value = "AdmEmsElecPgIndexController", description = "电网设施指标")
public class AdmEmsElecPgIndexController extends BaseController
{
    @Autowired
    private IAdmEmsElecPgIndexService elecPgIndexService;

    /**
     * 查询电网设施指标列表
     */
    @RequiresPermissions("ems:elecPgIndex:list")
    @GetMapping("/list")
    public TableDataInfo list(AdmEmsElecPgIndex elecPgIndex)
    {
        startPage();
        List<AdmEmsElecPgIndex> list = elecPgIndexService.selectAdmEmsElecPgIndexList(elecPgIndex);
        return getDataTable(list);
    }

    /**
     * 导出电网设施指标列表
     */
    @RequiresPermissions("ems:elecPgIndex:export")
    @Log(title = "电网设施指标", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, AdmEmsElecPgIndex elecPgIndex)
    {
        List<AdmEmsElecPgIndex> list = elecPgIndexService.selectAdmEmsElecPgIndexList(elecPgIndex);
        ExcelUtil<AdmEmsElecPgIndex> util = new ExcelUtil<AdmEmsElecPgIndex>(AdmEmsElecPgIndex.class);
        util.exportExcel(response, list, "电网设施指标数据");
    }

    /**
     * 获取电网设施指标详细信息
     */
    @RequiresPermissions("ems:elecPgIndex:query")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(elecPgIndexService.selectAdmEmsElecPgIndexById(id));
    }

    /**
     * 新增电网设施指标
     */
    @RequiresPermissions("ems:elecPgIndex:add")
    @Log(title = "电网设施指标", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody AdmEmsElecPgIndex elecPgIndex)
    {
        return toAjax(elecPgIndexService.insertAdmEmsElecPgIndex(elecPgIndex));
    }

    /**
     * 修改电网设施指标
     */
    @RequiresPermissions("ems:elecPgIndex:edit")
    @Log(title = "电网设施指标", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody AdmEmsElecPgIndex elecPgIndex)
    {
        return toAjax(elecPgIndexService.updateAdmEmsElecPgIndex(elecPgIndex));
    }

    /**
     * 删除电网设施指标
     */
    @RequiresPermissions("ems:elecPgIndex:remove")
    @Log(title = "电网设施指标", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(elecPgIndexService.deleteAdmEmsElecPgIndexByIds(ids));
    }
}
