package com.ruoyi.ems.mapper;

import com.ruoyi.ems.domain.EmsObjAttr;
import com.ruoyi.ems.domain.EmsObjAttrDto;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 能源对象属性Mapper接口
 *
 * @author ruoyi
 * @date 2024-09-23
 */
public interface EmsObjAttrMapper {
    /**
     * 查询能源对象属性
     *
     * @param id 能源对象属性主键
     * @return 能源对象属性
     */
    EmsObjAttr selectObjAttrById(Long id);

    /**
     * 查询能源对象属性列表
     *
     * @param objAttr 能源对象属性
     * @return 能源对象属性集合
     */
    List<EmsObjAttr> selectObjAttrList(EmsObjAttr objAttr);

    /**
     * 根据modelCode查询能源对象属性列表
     *
     * @param modelCode 设备模型代码
     * @return 能源对象能力集合
     */
    List<EmsObjAttr> selectByModelCode(@Param("modelCode") String modelCode);

    /**
     * 查询能源对象属性列表
     *
     * @param modelCode 设备模型代码
     * @param objType   设备类型
     * @param objCode   设备编码
     * @return 能源对象属性集合
     */
    List<EmsObjAttrDto> selectObjAttrDto(@Param("modelCode") String modelCode, @Param("objType") Integer objType,
        @Param("objCode") String objCode);

    /**
     * 新增能源对象属性
     *
     * @param objAttr 能源对象属性
     * @return 结果
     */
    int insertObjAttr(EmsObjAttr objAttr);

    /**
     * 批量新增能源对象属性
     *
     * @param list 能源对象能力
     * @return 结果
     */
    int insertBatch(List<EmsObjAttr> list);

    /**
     * 修改能源对象属性
     *
     * @param objAttr 能源对象属性
     * @return 结果
     */
    int updateObjAttr(EmsObjAttr objAttr);

    /**
     * 删除能源对象属性
     *
     * @param id 能源对象属性主键
     * @return 结果
     */
    int deleteObjAttrById(Long id);

    /**
     * 根据modelCode删除能源对象属性
     *
     * @param modelCode 设备模型代码
     * @return 结果
     */
    int deleteByModelCode(@Param("modelCode") String modelCode);

    /**
     * 批量删除能源对象属性
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    int deleteObjAttrByIds(Long[] ids);
}
