package com.ruoyi.ems.service.impl;

import com.ruoyi.ems.domain.EmsObjAttrValue;
import com.ruoyi.ems.mapper.EmsObjAttrValueMapper;
import com.ruoyi.ems.service.IEmsObjAttrValueService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 能源对象属性值Service业务层处理
 *
 * @author ruoyi
 * @date 2024-09-25
 */
@Service
public class EmsObjAttrValueServiceImpl implements IEmsObjAttrValueService {
    @Autowired
    private EmsObjAttrValueMapper objAttrValueMapper;

    /**
     * 查询能源对象属性值
     *
     * @param id 能源对象属性值主键
     * @return 能源对象属性值
     */
    @Override
    public EmsObjAttrValue selectObjAttrValueById(Long id) {
        return objAttrValueMapper.selectObjAttrValueById(id);
    }

    /**
     * 查询能源对象属性值列表
     *
     * @param objAttrValue 能源对象属性值
     * @return 能源对象属性值
     */
    @Override
    public List<EmsObjAttrValue> selectObjAttrValueList(EmsObjAttrValue objAttrValue) {
        return objAttrValueMapper.selectObjAttrValueList(objAttrValue);
    }

    @Override
    public List<EmsObjAttrValue> selectByObjCode(String modelCode, String objCode) {
        return objAttrValueMapper.selectByObjCode(modelCode, objCode);
    }

    /**
     * 新增/更新能源对象属性值
     *
     * @param objAttrValue 能源对象属性值
     * @return 结果
     */
    @Override
    public int mergeObjAttrValue(EmsObjAttrValue objAttrValue) {
        return objAttrValueMapper.replaceObjAttrValue(objAttrValue);
    }

    @Override
    public int insertBatch(List<EmsObjAttrValue> list) {
        EmsObjAttrValue objAttrValue = list.get(0);
        objAttrValueMapper.deleteByObjCode(objAttrValue.getModelCode(), objAttrValue.getObjCode());
        return objAttrValueMapper.insertBatch(list);
    }

    /**
     * 批量删除能源对象属性值
     *
     * @param ids 需要删除的能源对象属性值主键
     * @return 结果
     */
    @Override
    public int deleteObjAttrValueByIds(Long[] ids) {
        return objAttrValueMapper.deleteObjAttrValueByIds(ids);
    }

    /**
     * 删除能源对象属性值信息
     *
     * @param id 能源对象属性值主键
     * @return 结果
     */
    @Override
    public int deleteObjAttrValueById(Long id) {
        return objAttrValueMapper.deleteObjAttrValueById(id);
    }

    @Override
    public int deleteByObjCode(String modelCode, String objCode) {
        return objAttrValueMapper.deleteByObjCode(modelCode, objCode);
    }
}
